import { count } from "./Gamefield"
import { useGameProps } from "@hooks/useGameProps"
import useShips from "@hooks/useShips"
import {
  borderCN,
  cornerCN,
  fieldIndex,
  intersectingShip,
  isAlreadyHit,
  overlapsWithAnyBorder,
  shipProps,
  targetList,
} from "@lib/utils/helpers"
import { CSSProperties, useCallback } from "react"

type TilesType = {
  key: number
  isGameTile: boolean
  className: string
  x: number
  y: number
}

function BorderTiles() {
  const {
    DispatchAction,
    payload,
    mode,
    hits,
    target,
    targetPreview,
    mouseCursor,
    setTarget,
    setMouseCursor,
  } = useGameProps()
  const { ships, setShips, removeShip } = useShips()

  const settingTarget = useCallback(
    (isGameTile: boolean, x: number, y: number) => {
      if (payload?.game?.state === "running") {
        const list = targetList(targetPreview, mode)
        if (
          !isGameTile ||
          !list.filter(({ x, y }) => !isAlreadyHit(x, y, hits)).length
        )
          return
        if (target.show && target.x == x && target.y == y) {
          DispatchAction({
            action: "missile",
            ...target,
          })
          setTarget((t) => ({ ...t, show: false }))
        } else if (!overlapsWithAnyBorder(targetPreview, mode))
          setTarget({ show: true, x, y })
      } else if (
        payload?.game?.state === "starting" &&
        targetPreview.show &&
        !intersectingShip(ships, shipProps(ships, mode, targetPreview)).score
      ) {
        setMouseCursor((e) => ({ ...e, shouldShow: false }))
        setShips([...ships, shipProps(ships, mode, targetPreview)])
      }
    },
    [
      DispatchAction,
      hits,
      mode,
      payload?.game?.state,
      setMouseCursor,
      setShips,
      setTarget,
      ships,
      target,
      targetPreview,
    ]
  )
  let tilesProperties: TilesType[] = []

  for (let y = 0; y < count + 2; y++) {
    for (let x = 0; x < count + 2; x++) {
      const key = fieldIndex(count, x, y)
      const cornerReslt = cornerCN(count, x, y)
      const borderType = cornerReslt ? cornerReslt : borderCN(count, x, y)
      const isGameTile = x > 0 && x < count + 1 && y > 0 && y < count + 1
      const classNames = ["border-tile"]
      if (borderType) classNames.push("edge", borderType)
      if (isGameTile) classNames.push("game-tile")
      const className = classNames.join(" ")
      tilesProperties.push({
        key,
        className,
        isGameTile,
        x: x + 1,
        y: y + 1,
      })
    }
  }

  return (
    <>
      {tilesProperties.map(({ key, className, isGameTile, x, y }) => {
        return (
          <div
            key={key}
            className={className}
            style={{ "--x": x, "--y": y } as CSSProperties}
            onClick={() => {
              if (payload?.game?.state === "running") {
                settingTarget(isGameTile, x, y)
              } else if (payload?.game?.state === "starting") {
                const { index } = intersectingShip(ships, {
                  ...mouseCursor,
                  size: 1,
                  variant: 0,
                  orientation: "h",
                })
                if (typeof index === "undefined")
                  settingTarget(isGameTile, x, y)
                else {
                  const ship = ships[index]
                  useGameProps.setState({ mode: ship.size - 2 })
                  removeShip(ship)
                  setMouseCursor((e) => ({ ...e, shouldShow: true }))
                }
              }
            }}
            onMouseEnter={() =>
              setMouseCursor({
                x,
                y,
                shouldShow:
                  isGameTile &&
                  (payload?.game?.state === "starting"
                    ? intersectingShip(
                        ships,
                        shipProps(ships, mode, {
                          x,
                          y,
                          orientation: targetPreview.orientation,
                        }),
                        true
                      ).score < 2
                    : true),
              })
            }
          ></div>
        )
      })}
    </>
  )
}

export default BorderTiles
